@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/connection.c,v 1.358.2.38 2009/11/22 16:28:23 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2009 The OpenLDAP Foundation.
+ * Copyright 1998-2011 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -39,6 +39,10 @@
 #include "lutil.h"
 #include "slap.h"
 
+#ifdef LDAP_CONNECTIONLESS
+#include "../../libraries/liblber/lber-int.h"	/* ber_int_sb_read() */
+#endif
+
 #ifdef LDAP_SLAPI
 #include "slapi/slapi.h"
 #endif
@@ -221,6 +225,7 @@ int connections_timeout_idle(time_t now)
 		 */
 		if(( c->c_n_ops_executing && !c->c_writewaiter)
 			|| c->c_conn_state == SLAP_C_CLIENT ) {
+			connection_done( c );
 			continue;
 		}
 
@@ -239,16 +244,40 @@ int connections_timeout_idle(time_t now)
 				connection_closing( c, "writetimeout" );
 				connection_close( c );
 				i++;
+				continue;
 			}
 		}
+		connection_done( c );
 	}
-	connection_done( c );
 	if ( old && !writers )
 		slapd_clr_writetime( old );
 
 	return i;
 }
 
+/* Drop all client connections */
+void connections_drop()
+{
+	Connection* c;
+	int connindex;
+
+	for( c = connection_first( &connindex );
+		c != NULL;
+		c = connection_next( c, &connindex ) )
+	{
+		/* Don't close a slow-running request or a persistent
+		 * outbound connection.
+		 */
+		if(( c->c_n_ops_executing && !c->c_writewaiter)
+			|| c->c_conn_state == SLAP_C_CLIENT ) {
+			connection_done( c );
+			continue;
+		}
+		connection_closing( c, "dropping" );
+		connection_close( c );
+	}
+}
+
 static Connection* connection_get( ber_socket_t s )
 {
 	Connection *c;
@@ -272,12 +301,11 @@ static Connection* connection_get( ber_socket_t s )
 		if( c->c_struct_state != SLAP_C_USED ) {
 			/* connection must have been closed due to resched */
 
-			assert( c->c_conn_state == SLAP_C_INVALID );
-			assert( c->c_sd == AC_SOCKET_INVALID );
-
 			Debug( LDAP_DEBUG_CONNS,
 				"connection_get(%d): connection not used\n",
 				s, 0, 0 );
+			assert( c->c_conn_state == SLAP_C_INVALID );
+			assert( c->c_sd == AC_SOCKET_INVALID );
 
 			ldap_pvt_thread_mutex_unlock( &c->c_mutex );
 			return NULL;
@@ -699,7 +727,6 @@ static void connection_abandon( Connection *c )
 
 	Operation *o, *next, op = {0};
 	Opheader ohdr = {0};
-	SlapReply rs = {0};
 
 	op.o_hdr = &ohdr;
 	op.o_conn = c;
@@ -707,6 +734,8 @@ static void connection_abandon( Connection *c )
 	op.o_tag = LDAP_REQ_ABANDON;
 
 	for ( o = LDAP_STAILQ_FIRST( &c->c_ops ); o; o=next ) {
+		SlapReply rs = {REP_RESULT};
+
 		next = LDAP_STAILQ_NEXT( o, o_next );
 		op.orn_msgid = o->o_msgid;
 		o->o_abandon = 1;
@@ -1362,8 +1391,8 @@ connection_read( ber_socket_t s, conn_readinfo *cri )
 			    c->c_connid, (int) s, c->c_tls_ssf, c->c_ssf, 0 );
 			slap_sasl_external( c, c->c_tls_ssf, &authid );
 			if ( authid.bv_val ) free( authid.bv_val );
-		} else if ( rc == 1 ) {	/* need to retry */
-			slapd_set_read( s, 0 );
+		} else if ( rc == 1 && ber_sockbuf_ctrl( c->c_sb,
+			LBER_SB_OPT_NEEDS_WRITE, NULL )) {	/* need to retry */
 			slapd_set_write( s, 1 );
 			connection_return( c );
 			return 0;
@@ -1522,8 +1551,8 @@ connection_input( Connection *conn , conn_readinfo *cri )
 #ifdef LDAP_CONNECTIONLESS
 	if( conn->c_is_udp ) {
 		if( tag == LBER_OCTETSTRING ) {
-			ber_get_stringa( ber, &cdn );
-			tag = ber_peek_tag(ber, &len);
+			if ( (tag = ber_get_stringa( ber, &cdn )) != LBER_ERROR )
+				tag = ber_peek_tag( ber, &len );
 		}
 		if( tag != LDAP_REQ_ABANDON && tag != LDAP_REQ_SEARCH ) {
 			Debug( LDAP_DEBUG_ANY, "invalid req for UDP 0x%lx\n", tag, 0, 0 );
